#include <windows.h>
#include <stdio.h>
#include <string.h>
#include <conio.h>

#include "aldr9.h"
#include "main.h"

////////////////////////////////////////////////////////////////////

#include "OlhaFullFax.h"

#pragma comment( lib, "OlhaFullFax.lib" )

///////////////////////////////////////////////////////////////////////

void PrintUsage( char *ProgName)
{	
	char *Ptr;

	if(( Ptr= strrchr( ProgName, '\\'))== NULL)
		Ptr= ProgName;
	printf( "FAX receive simple test for Alder9 board (ISA&PCI) v1.0\n");
	printf( "Usage: %s [/B<BasePort>][/Q<IrqNum>][/L<Line>][/T][/I][/P][/T]<FileName.bmp>\n", Ptr);
	printf( "  /B<BasePort> Base port address in hex ( Find PCI board if not defined)\n");
	printf( "  /Q<IrqNum> IRQ number( 11 by default)\n");
	printf( "  /L<Line> Line number( 0 by default)\n");
	printf( "  /I Internal extention line( by default)\n");
	printf( "  /E External town line\n");
	printf( "  /K<string> Key string for legal fax use.\n");
	printf( "  <FileName.bmp> - BMP file for receive form remote fax\n");
	printf( "\n\n");
}

DWORD ShowStat( AlderReceiveFaxData *afd)
{	
	static DWORD NewStatFax, OldStatFax= 0;
	char pszStatConnect[64]= "";

	NewStatFax= (DWORD)AlderGetReceiverStatus( afd);
	if( OldStatFax== NewStatFax) 
		return NewStatFax;
	OldStatFax= NewStatFax;
	switch( NewStatFax)
	{	
		case FaxReceiveSendDIS:   
		    strcpy( pszStatConnect," Sending DIS ");
			break;
		case FaxReceiveWaitDCS:
			strcpy( pszStatConnect," Wait for DCS ");
			break;
		case FaxReceiveTren2400:
            strcpy( pszStatConnect," Training 2400 ");
			break;
		case FaxReceiveTren4800:
            strcpy( pszStatConnect," Training 4800 ");
			break;
		case FaxReceiveTren7200:
            strcpy( pszStatConnect," Training 7200 ");
			break;
		case FaxReceiveTren9600:
            strcpy( pszStatConnect," Training 9600 ");
			break;
		case FaxReceive2400:
            strcpy( pszStatConnect," Receiving 2400 ");
			break;
		case FaxReceive4800:
            strcpy( pszStatConnect," Receiving 4800 ");
			break;
		case FaxReceive7200:
            strcpy( pszStatConnect," Receiving 7200 ");
			break;
		case FaxReceive9600:
            strcpy( pszStatConnect," Receiving 9600 ");
			break;
		case FaxReceiveWaitPMC:
            strcpy( pszStatConnect," Waiting for PMC ");
			break;
		case FaxReceiveSendPIN:
            strcpy( pszStatConnect," Sending PIN ");
			break;
		case FaxReceiveSendMCF:
            strcpy(  pszStatConnect," Sending MCF ");
			break;
		case FaxReceiveSendFTT:
            strcpy( pszStatConnect," Failure to training ");
			break;
		case FaxReceiveSendCFR:
            strcpy( pszStatConnect," Training OK ");
			break;
		case FaxReceiveSendCRP:
            strcpy( pszStatConnect," Sending CRP ");
			break;
	}
	printf(" Status: %s\n", pszStatConnect);
	return NewStatFax;
}

void ShowFinalStatus( AlderReceiveFaxData *afd)
{
	switch( afd->dwFinalStatus)
	{
		case FaxReceiveNoError:		
			printf("OK!\n");
			break;
		 case FaxReceiveSystemError:
			printf("There was a system error during receiving. GetLastError:%d\n", 
					afd->dwSystemError);
			break;
		case FaxReceiveDriverError:			
			printf(" N:%d. The driver has returned a error number %04X\n",
					afd->dwUserMessNumberStop, afd->dwDrvError);
			break;
		case FaxReceiveDeviceIoError:		
			printf( "DeviceIOControl was finished unsuccessfully. GetLastError:%d\n", 
					afd->dwSystemError);
			break;
		case FaxReceiveMemoryFault:			
			printf("Error at allocation of memory.\n");
			break;
		case FaxReceiveAlreadyStarted:		
			printf("The receiver is already started.\n");
			break;
		case FaxReceiveFileWriteError:
			printf("Error at record of a out file\n");
			break;
		case FaxReceiveNoFaxPages:			
			printf("In an input file there are no facsimile sheets.\n");
			break;
		case FaxReceiveTooManyLines:
			printf("Too large number of a board / channel.\n");
			break;
		case FaxReceiveDriverVersionFault:	
			printf("The wrong version of the driver.\n");
			break;
		case FaxReceiveFileCorrupt:
			printf("The input file is corrupted.\n");
			break;
		case FaxReceiveMustBeRun:
			printf("The receiver should be already started.\n");
			break;
		case FaxReceiveUncorrectVersion:
			printf( "Uncorrect Dll version!\n");
			break;
		default:
			printf("Ungecognized error.\n");
			break;
	}
}

void main( int argc, char **argv)
{	
	DWORD dwBasePort= 0, dwBoardHandle, dwIrqFreq= 9, dwIrq= 11;
	DWORD dwLine= 0;
	DWORD StartTicks, TotalTicks, dwI;
	DWORD dwBoardType, dwDllMajor, dwDllMinor;
	HANDLE hDrv;
	char *pszFileName, *pszKey;
	bool blInternalLine= true;
	AlderReceiveFaxData afd;

	PrintUsage( argv[0]);

	if(( hDrv= hOpenDriver())== NULL)
	{	
		printf( " Unable open the driver. System error: %d\n\n", GetLastError());
		return;
	}
	blGetDriverVersion( hDrv);
	// Load command strings params
	if( argc> 1)
	{
		for( dwI= 1; dwI< (DWORD)argc; dwI++)
		{	
			// Get BasePort
			if( strncmp( argv[dwI], "/B", 2)== 0)
			{
				sscanf( argv[dwI]+ 2, "%X", &dwBasePort);
				if( blCheckBasePort( dwBasePort)== false)
				{
					printf( "Incorrect parameter \'Base Port: %04X\'\n", dwBasePort);
					return;
				}
				continue;
			}
			// Get IRQ number
			if( strncmp( argv[dwI], "/Q", 2)== 0)
			{
				dwIrq= atoi( argv[dwI]+2);
				if( blCheckLegalIrq( dwIrq)== false)
				{
					printf( "Incorrect parameter \'Irq: %d\'\n", dwIrq);
					return;
				}
				continue;
			}
			// Get Line number
			if( strncmp( argv[dwI], "/L", 2)== 0)
			{
				dwLine= atoi( argv[dwI]+ 2);
				if( dwLine> 15)
				{
					printf( "Incorrect parameter \'Line: %d\'\n", dwLine);
					return;
				}
				continue;
			}
			// Get Line Type - Town (external) or Extention (internal)
			if( strncmp( argv[dwI], "/I", 2)== 0)
			{
				blInternalLine= true;
				continue;
			}
			if( strncmp( argv[dwI], "/E", 2)== 0)
			{
				blInternalLine= false;
				continue;
			}
			if( strncmp( argv[dwI], "/K", 2)== 0)
			{
				pszKey= &argv[dwI][2];
				continue;
			}
			pszFileName= argv[dwI];		
		}
	}
	if( dwBasePort== 0)
	{	
		// Find PCI board
		if( blGetPciBoardInfo( hDrv, &dwBasePort, &dwBoardType)== false)
			goto EndProg;
		if( dwBasePort== 0)
		{	
			printf( "No Pci board found\n\n");
			goto EndProg;
		}
		printf( "PCI board found on %04X\n", dwBasePort);
	}
	else
	{	
		if( blGetBoardType( hDrv, dwBasePort, &dwBoardType)== false)
			goto EndProg;
		printf( "BoardType is: %04X\n", dwBoardType);
		if(( dwBoardType!= Alder9Board) && ( dwBoardType!= Alder10Board))
		{	
			printf( "Alder9 or Alder10 board not founded on port %04X\n", dwBasePort);
			goto EndProg;
		}
		switch( dwBoardType)
		{
			case Alder9Board:
				printf( "Alder9 ISA board found on %04X\n", dwBasePort);
				break;
			case Alder10Board:
				printf( "Alder10 ISA board found on %04X\n", dwBasePort);
				break;
			default:
				break;
		}
	}
	if( blOpenBoard( hDrv, dwBoardType, dwBasePort, &dwBoardHandle)== false)
		goto EndProg;
	if( blStartBoards( hDrv, dwIrq)== false)
		goto CloseBoard;
	StartTicks= GetTickCount();
	if( blInternalLine== false)
	{
		if( blSetPhoneParams( hDrv, dwBoardHandle, dwLine, 8, 1)== false)
			goto StopBoard;
	}
	else
	{
		if( blSetPhoneParams( hDrv, dwBoardHandle, dwLine, 4, 1)== false)
			goto StopBoard;
	}
	printf( "Selected line is %d\n", dwLine);
	AlderGetDllVersion( &dwDllMajor, &dwDllMinor);
	printf( " DLL version is %d.%d\n", dwDllMajor, dwDllMinor);
	printf( "Press ESC for exit...\n");

///*******************************************************************
	//     AlderReceiveFaxData
	//  
	ZeroMemory( &afd, sizeof( AlderReceiveFaxData));
	afd.cbSize= sizeof( AlderReceiveFaxData);
	//       
	lstrcpy( afd.pszKey, pszKey);
	//  .     
	afd.hcDrv= hDrv;
	//  
    afd.dwBoardHandle= dwBoardHandle;
	//      
	afd.dwLine= dwLine;
	//    -
	afd.EndFaxReceivFunc= NULL;
	//       
	afd.dwUserMessNumberStop= 0;
///*******************************************************************
	//      
	// BMP     
	lstrcpy( afd.pszFaxFileName, pszFileName);
	if( blWaitSignal( hDrv, dwBoardHandle, dwLine, drvAlderLineRing, 30)== false)
		goto HookOff;
	if( blSetHook( hDrv, dwBoardHandle, dwLine, true)== false)
		goto StopBoard;
	if( blWaitSignal( hDrv, dwBoardHandle, dwLine, drvAlderLineIncoming, 10)== false)
		goto HookOff;
	AlderOpenFaxReceiver( &afd);
	do
	{	
		//    (BUSY)
		if( blCheckSignal( hDrv, dwBoardHandle, dwLine, drvAlderLineBusy)== true)
		{
			printf( " BUSY found!!!\n");
			break;
		}
		printf("\r");
		if( kbhit()!=0) 
		{	
			if( getch()== 0x1B)
				break;
		}
		Sleep( 50);
	} while( ShowStat( &afd)!= FaxReceiveOff);
	ShowFinalStatus( &afd);
	AlderCloseFaxReceiver( &afd);
	printf( "End Of Fax Receive\n\n");
HookOff:
	blSetHook( hDrv, dwBoardHandle, dwLine, false);
	Sleep( 100);
StopBoard:
	blStopBoards( hDrv);
CloseBoard:
	blCloseBoard( hDrv, dwBoardHandle);
	TotalTicks= GetTickCount()- StartTicks;
	printf( "Total processing time: %d.%d\n", TotalTicks/1000, TotalTicks%1000);
EndProg:
	CloseDriver( hDrv);
}
