#include <windows.h>
#include <stdio.h>
#include <conio.h>

#include "aldr9.h"
#include "aldr9err.h"
#include "main.h"

DWORD dwGetLastError( HANDLE hDrv)
{	
	DWORD dwReaded, dwError= 0;

	DeviceIoControl( hDrv, drvA9GetLastError, 
					 NULL, 0,
					 &dwError, sizeof( DWORD),
					 &dwReaded, NULL);
	return dwError;
}

bool blShowErr( HANDLE hDrv, char *szFuncName, DWORD dwDrvError, DWORD dwSysError)
{	
	DWORD dwErr;

	if( dwDrvError== 0)
	{	
		printf( "Routine \'%s\' call error. System error: %d.\n", 
				szFuncName, dwSysError);
		switch( dwSysError)
		{	
			case ERROR_INSUFFICIENT_BUFFER:
				printf( " Input or output buffer for IOCTL too small.\n\n");
				return false;
			case ERROR_INVALID_HANDLE:
				printf( " Invalid board handle.\n\n");
				return false;
			case ERROR_BUSY:
				printf( " Resource ( port, board or IRQ) busy.\n\n");
				return false;
			case ERROR_NOT_ENOUGH_MEMORY:
				printf( " Driver memory allocate error.\n\n");
				return false;
			case ERROR_RESOURCE_TYPE_NOT_FOUND:
				printf( " Incorrect board loader.\n\n");
				return false;
			case ERROR_NOT_READY:
				printf( " Device not functionally. Device init error.\n\n");
				return false;
			case ERROR_INVALID_FUNCTION:
				printf( " Invalid function for this line type.\n\n");
				return true;
			case ERROR_NOT_SUPPORTED:
				printf( " This function not suppoted by this driver.\n\n");
				return false;
			case ERROR_LOCK_VIOLATION:
				printf( " Line is locked! Operation not performed.\n\n");
				return true;
			case ERROR_INVALID_PARAMETER:
				printf( " Invalid parameters for IOCTL.\n\n");
				return false;
			case ERROR_IO_DEVICE:
				printf( " Driver panic - fatal error.\n");
				//dwDrvError= dwGetLastError( hDrv);
				dwErr= dwGetLastError( hDrv);
				switch( dwErr)
				{
					case AlderIncorrectBoardFifo:
						printf( "Board error. Incorrect board FIFO buffer.\n\n");
						break;
					case AlderIncorrectCmdFromBoard:
						printf( "Board error. Incorrect data from board.\n\n");
						break;
					case AlderIncorrectLineFromBoard:
						printf( "Board error. Incorrect line number from board to driver.\n\n");
						break;
					case AlderIncorrectCmdToBoard:
						printf( "Driver error. Incorrect command from driver to board.\n\n");
						break;
					case AlderChangeIrqError:
						printf( "Driver error. Error for IRQ change!\n\n");
						break;
					default:
						printf( " Unrecognized error %d...\n\n", dwErr);
						break;
				}
				return false;
			default:
				printf( " Unrecognized error..\n\n");
				return false;
		}
		
	}
	return false;
}

bool blCheckBasePort( DWORD dwBasePort)
{	
	DWORD dwI, dwBeginAddr;

	for( dwI= 0, dwBeginAddr= 0x200; dwI< 32; dwI++, dwBeginAddr+= 4)
	{	
		if( dwBasePort== dwBeginAddr)
			return true;
	}
	return false;
}

bool blCheckLegalIrq( DWORD dwIrq)
{	
	DWORD dwI, lpdwLegalIrq[8]= { 3, 5, 7, 9, 10, 11, 12, 15};

	for( dwI= 0; dwI< 8; dwI++)
	{	
		if( lpdwLegalIrq[dwI]== dwIrq)
			return true;
	}
	return false;
}

HANDLE hOpenDriver()
{	
	char DriverName[128];
	HANDLE hDrv;

	lstrcpy( DriverName, "\\\\.\\ALDER9");
	//   
	if(( hDrv= CreateFile( DriverName,
							GENERIC_READ | GENERIC_WRITE,
							FILE_SHARE_READ | FILE_SHARE_WRITE,
							NULL,
							OPEN_EXISTING,
							FILE_ATTRIBUTE_NORMAL,
							NULL))== INVALID_HANDLE_VALUE)
	{
		DeleteFile("\\\\.\\ALDER9");
		DeleteFile("\\\\.\\ALDER9");
		DeleteFile("\\\\.\\ALDER9");
		DeleteFile("\\\\.\\ALDER9");
		lstrcpy( DriverName, "\\\\.\\ALDER9.VXD");
		if(( hDrv= CreateFile( DriverName,
								GENERIC_READ | GENERIC_WRITE,
								FILE_SHARE_READ | FILE_SHARE_WRITE,
								NULL,
								OPEN_EXISTING,
								FILE_ATTRIBUTE_NORMAL,
								NULL))== INVALID_HANDLE_VALUE)
			return NULL;
	}
	return hDrv;
}

void CloseDriver( HANDLE hDrv)
{	
	if( hDrv!= NULL)
	{	
		CloseHandle( hDrv);
		hDrv= NULL;
	}
}

bool blGetDriverVersion( HANDLE hDrv)
{	
	DWORD dwReaded;
	DriverVersion Version;

	if( DeviceIoControl( hDrv, drvA9GetVersion, 
						 NULL, 0,
						 &Version, sizeof( DriverVersion),
						 &dwReaded, NULL)== 0)
		return blShowErr( hDrv, "blGetVersion", 0, GetLastError());
	printf( "Driver version is %d.%d.%d\n", 
			Version.dwMajor, Version.dwMinor, Version.dwLoader);
	return true;
}

bool blGetBoardType( HANDLE hDrv, DWORD dwBasePort, DWORD *lpdwType)
{	
	DWORD dwReaded;

	if( DeviceIoControl( hDrv, drvA9GetBoardType, 
						 &dwBasePort, sizeof( DWORD),
						 lpdwType, sizeof( DWORD),
						 &dwReaded, NULL)== 0)
		return blShowErr( hDrv, "blGetBoardType", 0, GetLastError());
	return true;
}

bool blOpenBoard( HANDLE hDrv, DWORD dwBoardType, DWORD dwBasePort, DWORD *lpdwBoardHandle)
{	
	HANDLE lh;
	BoardLoadData BrdLoad;
	BoardInfo BrdInfo;
	DWORD dwReaded; //, dwI;
	char pszLoaderName[32];

	ZeroMemory( &BrdLoad, sizeof( BoardLoadData));
	ZeroMemory( &BrdInfo, sizeof( BoardInfo));

	//     
	switch( dwBoardType)
	{
		case Alder9Board:
			lstrcpy( pszLoaderName, "ALDR9A.BIN");
			break;
		case Alder10Board:
			lstrcpy( pszLoaderName, "ALDR10A.BIN");
			break;
		case Alder10LPBoard:
			lstrcpy( pszLoaderName, "ALDR10LP.BIN");
			break;
		default:
			break;
	}
	if(( lh= CreateFile( pszLoaderName, GENERIC_READ, 0, NULL,
						 OPEN_EXISTING, 0, NULL))== INVALID_HANDLE_VALUE)
		return blShowErr( hDrv, "OpenBoard", 0, GetLastError());
	BrdLoad.dwLoadSize= GetFileSize( lh, NULL);
	if(( BrdLoad.lpbLoader= ( BYTE *)GlobalAlloc( GMEM_FIXED, BrdLoad.dwLoadSize))== NULL)
	{   
		CloseHandle( lh);
		return blShowErr( hDrv, "OpenBoard", 0, GetLastError());
	}
	if( ReadFile( lh, BrdLoad.lpbLoader, BrdLoad.dwLoadSize, &dwReaded, NULL)== 0)
	{	
		CloseHandle( lh);
		GlobalFree( BrdLoad.lpbLoader);
		return blShowErr( hDrv, "OpenBoard", 0, GetLastError());
	}
	CloseHandle( lh);
	//    
	BrdLoad.dwBasePort= dwBasePort;

	if( DeviceIoControl( hDrv, drvA9OpenBoard,
						 &BrdLoad, sizeof( BoardLoadData),
						 &BrdInfo, sizeof( BoardInfo),
						 &dwReaded, NULL)== 0)
	{	
		GlobalFree( BrdLoad.lpbLoader);
		return blShowErr( hDrv, "OpenBoard", 0, GetLastError());;
	}
	GlobalFree( BrdLoad.lpbLoader);
	*lpdwBoardHandle= BrdInfo.dwBoardHandle;
	printf( "Factory board number:%d Loader version:%d E1Version: %d.%d\n", 
			 BrdInfo.dwFactoryNumber, BrdInfo.dwLoaderVersion,
			 BrdInfo.dwE1MajorVersion, BrdInfo.dwE1MinorVersion);
/*
	printf( "ResourceMode:%d Lines:%d Voice lines:%d\n", 
			BrdInfo.dwResourceMode, BrdInfo.dwLines, BrdInfo.dwVoiceLines);
	printf( "* Line types:\n");
	for( dwI= 0; dwI< BrdInfo.dwLines; dwI++)
	{	
		if(((dwI+1)%32)!= 0)
			printf( " %d", BrdInfo.lpbLinesTypes[dwI]);
		else
			printf( " %d\n", BrdInfo.lpbLinesTypes[dwI]);
	}
	printf( "\n");
	printf( "* Possible Line types:\n");
	for( dwI= 0; dwI< BrdInfo.dwLines; dwI++)
	{	
		if(((dwI+1)%32)!= 0)
			printf( " %1X", BrdInfo.lpbPossibleLinesTypes[dwI]);
		else
			printf( " %1X\n", BrdInfo.lpbPossibleLinesTypes[dwI]);
	}
	printf( "\n");
*/

	return true;
}

bool blCloseBoard( HANDLE hDrv, DWORD dwBoardHandle)
{	
	DWORD dwReaded;

	if( DeviceIoControl( hDrv, drvA9CloseBoard, 
						 &dwBoardHandle, sizeof( DWORD),
						 NULL, 0,
						 &dwReaded, NULL)== 0)
		return blShowErr( hDrv, "blCloseBoard", 0, GetLastError());
	return true;
}

bool blStartBoards( HANDLE hDrv, DWORD dwIrq)
{	
	DWORD dwReaded;

	if( DeviceIoControl( hDrv, drvA9StartBoards, 
						 &dwIrq, sizeof( DWORD),
						 NULL, 0,
						 &dwReaded, NULL)== 0)
		return blShowErr( hDrv, "blStartBoards", 0, GetLastError());
	return true;
}

bool blStopBoards( HANDLE hDrv)
{	
	DWORD dwReaded;

	if( DeviceIoControl( hDrv, drvA9StopBoards, 
						 NULL, 0, NULL, 0,
						 &dwReaded, NULL)== 0)
		return blShowErr( hDrv, "blStopBoards", 0, GetLastError());
	return true;
}

bool blGetLineStatus( HANDLE hDrv, DWORD dwBoardHandle, 
					  DWORD dwLine, A9LineStatus *ls)
{	
	DWORD dwReaded;
	LinesStatusParams lsp;

	lsp.dwBoardHandle= dwBoardHandle;
	lsp.dwBeginLine= dwLine;
	lsp.dwAmountLines= 1;
	if( DeviceIoControl( hDrv, drvA9GetLinesStatus,
						 &lsp, sizeof( LinesStatusParams),
						 ls, sizeof( A9LineStatus),
						 &dwReaded, NULL)== 0)
		return blShowErr( hDrv, "blGetLineStatus", 0, GetLastError());
	return true;
}

bool blSetPhoneParams( HANDLE hDrv, DWORD dwBoardHandle, DWORD dwLine, 
					   DWORD dwHookLevel, DWORD dwRingLevel)
{
	DWORD dwReaded;
	PhoneLineParams plp;

	plp.dwBoardHandle= dwBoardHandle;
	plp.dwLine= dwLine;
	plp.dwHookLevel= dwHookLevel;
	plp.dwRingLevel= dwRingLevel;
	if( DeviceIoControl( hDrv, drvA9SetPhoneParams,
						 &plp, sizeof( PhoneLineParams),
						 0, 0, &dwReaded, NULL)== 0)
		return blShowErr( hDrv, "blSetPhoneParams", 0, GetLastError());
	return true;
}

bool blSetHook( HANDLE hDrv, DWORD dwBoardHandle, DWORD dwLine, bool blMode)
{	
	DWORD dwReaded;
	HookParams hp;

	hp.dwBoardHandle= dwBoardHandle;
	hp.dwLine= dwLine;
	hp.dwMode= (DWORD)blMode;
	if( DeviceIoControl( hDrv, drvA9SetHook,
						 &hp, sizeof( HookParams),
						 0, 0, &dwReaded, NULL)== 0)
		return blShowErr( hDrv, "blSetHook", 0, GetLastError());
	return true;
}

bool blGetPciBoardInfo( HANDLE hDrv, DWORD *lpdwBasePort, DWORD *lpdwBoardType)
{	
	DWORD dwReaded, dwAmount, dwI;
	PciBoardData pbd[10];

	*lpdwBasePort= 0;
	if( DeviceIoControl( hDrv, drvA9GetPciBoardAmount,
						 0, 0,  
						 &dwAmount, sizeof( DWORD),
						 &dwReaded, NULL)== 0)
		return blShowErr( hDrv, "blGetPciBoardAmount", 0, GetLastError());

	if( dwAmount== 0)
	{
		printf( "No PCI boards.\n");
		return true;
	}
	printf( " %d PCI board found\n", dwAmount);
	if( DeviceIoControl( hDrv, drvA9GetPciBoardInfo,
						 0, 0,  
						 pbd, sizeof( PciBoardData)*dwAmount,
						 &dwReaded, NULL)== 0)
		return blShowErr( hDrv, "blGetPciBoardInfo", 0, GetLastError());
	for( dwI= 0; dwI< dwAmount; dwI++)
	{	
		printf( "Board:%d BaseAddr:%04X CtrlAddr:%04X Irq:%d PCIBus: %d\n", 
				dwI, pbd[dwI].dwBasePort, pbd[dwI].dwCtrlPort, 
				pbd[dwI].dwIrq, pbd[dwI].dwBusNumber);
	}
	*lpdwBasePort= pbd[0].dwBasePort;
	*lpdwBoardType= pbd[0].dwBoardType;
	return true;
}

bool blWaitSignal( HANDLE hDrv, DWORD dwBoardHandle, DWORD dwLine, 
					DWORD dwSignal, int TimeOutInSec)
{	
	A9LineStatus ls;

	for( int i= 0; i< TimeOutInSec*10; i++)
	{	
		if( kbhit()!= 0)
		{	
			if( getch()== 0x1B)
			{	
				printf( "User break!\n");	
				return false;
			}
		}
		if( blGetLineStatus( hDrv, dwBoardHandle, dwLine, &ls)== false)
			return false;
		if(( ls.dwChanged & dwSignal)!= 0)
		{	
			if(( ls.dwStatus & dwSignal)!= 0)
			{
				switch( dwSignal)
				{
					case drvAlderLineDialTone:
						printf( "DIALTONE signal found...\n");
						break;
					case drvAlderLineBusy:
						printf( "BUSY signal found...\n");
						break;
					case drvAlderLineFAX:
						printf( "FAX signal found...\n");
						break;
					case drvAlderLineRing:
						printf( "RING found...\n");
						break;
				}
				return true;
			}
		}
		Sleep( 100);
	}
	switch( dwSignal)
	{
		case drvAlderLineDialTone:
			printf( "DIALTONE signal not found...\n");
			break;
		case drvAlderLineBusy:
			printf( "BUSY signal not found...\n");
			break;
		case drvAlderLineFAX:
			printf( "FAX signal not found...\n");
			break;
	}
	return false;
}

bool blCheckSignal( HANDLE hDrv, DWORD dwBoardHandle, DWORD dwLine, 
					DWORD dwSignal)
{	
	A9LineStatus ls;

	if( blGetLineStatus( hDrv, dwBoardHandle, dwLine, &ls)== false)
		return false;
	if(( ls.dwChanged & dwSignal)!= 0)
	{	
		if(( ls.dwStatus & dwSignal)!= 0)
			return true;
	}
	return false;
}

