#include <windows.h>
#include <stdio.h>
#include <conio.h>

#include "aldr9.h"
#include "aldr9err.h"
#include "main.h"

#pragma pack(4)
typedef struct _tagDebugData
{
	DWORD dwIrqCount;
	DWORD dwDebug;
	DWORD dwIDMA;
	DWORD dwPack;	
	DWORD dwLoadTrace;
	WORD  lpwPack[40];
} DebugData;
#pragma pack()

bool blGetDebugData( HANDLE hDrv)
{
	DWORD dwReaded, dwI;
	DebugData dd;

	if( DeviceIoControl( hDrv, drvA9GetDebugData,
						 NULL, 0,
						 &dd, sizeof( DebugData),
						 &dwReaded, NULL)== 0)
		return blShowErr( hDrv, "blGetDebugData", 0, GetLastError());
	printf( " dwIrqCount: %d\n", dd.dwIrqCount);
	printf( " dwDebug:    %d\n", dd.dwDebug);
	printf( " dwIDMA:     %d\n", dd.dwIDMA);
	printf( " dwPack:     %08X\n", dd.dwPack);
	printf( " dwLoadTrace:%d\n", dd.dwLoadTrace);
	printf( " lpdwPack: ");
	for( dwI= 0; dwI< 20; dwI++)
		printf( "%04X ", dd.lpwPack[dwI]);
	printf( "\n");
	return true;
}

bool blWriteString( HANDLE fh, char *pszText)
{	DWORD Writed;

	if( WriteFile( fh, pszText, lstrlen( pszText), &Writed, NULL)== 0)
	{	
		printf( " Error write file. System error: %d\n\n", GetLastError());
		return false;
	}
	return true;
}

typedef struct _tagAdspLinesTable
{	
	DWORD		dwIDMAID;
	DWORD		dwFirstLine;
	DWORD		dwLinesAmount;
	DWORD 		dwVoiceLinesAmount;
} AdspLinesTable;

bool blGetIdmaTable( HANDLE hDrv, DWORD dwBoardHandle)
{	DWORD dwReaded, dwError= 0;
	AdspLinesTable alt[12];
	WORD lpwLinesTable[256];
	char pszLogFile[32]= "tables.log";
	char pszText[128];
	HANDLE hLog;
	DWORD dwI;

	if( DeviceIoControl( hDrv, drvA9GetIDMATable, 
						 &dwBoardHandle, sizeof( DWORD),
						 alt, sizeof( AdspLinesTable)*12,
						 &dwReaded, NULL)== 0)
		return blShowErr( hDrv, "blGetIdmaTable", 0, GetLastError());

	if( DeviceIoControl( hDrv, drvA9GetLinesTable, 
						 &dwBoardHandle, sizeof( DWORD),
						 lpwLinesTable, sizeof( WORD)*256,
						 &dwReaded, NULL)== 0)
		return blShowErr( hDrv, "blGetLinesTable", 0, GetLastError());

	if(( hLog= CreateFile( pszLogFile, GENERIC_WRITE, 
						   0, NULL, CREATE_ALWAYS, 
						   0, NULL))== INVALID_HANDLE_VALUE)
	{	
		printf( "Error create file \'%s\'. System error: %d.\n\n", 
				pszLogFile, GetLastError());
		return false;
	}

	wsprintf( pszText, "*** Driver IDMA table\r\n");
	blWriteString( hLog, pszText);

	for( dwI= 0; dwI< 12; dwI++)
	{	
		wsprintf( pszText, "IDMAID:%04X FirstLine:%3d LineAmount:%3d VoiceAmount:%3d\r\n",
				  alt[dwI].dwIDMAID, alt[dwI].dwFirstLine, alt[dwI].dwLinesAmount,
				  alt[dwI].dwVoiceLinesAmount);
		if( blWriteString( hLog, pszText)== false)
			break;
	}

	wsprintf( pszText, "\n*** Driver lines coding table\r\n");
	blWriteString( hLog, pszText);
	wsprintf( pszText, "*** Interface lines coding table\r\n");
	blWriteString( hLog, pszText);
	for( dwI= 0; dwI< 128; dwI++)
	{	
		if((( dwI+1)%16)== 0)
			wsprintf( pszText, "%04X\r\n", lpwLinesTable[dwI]);
		else
			wsprintf( pszText, "%04X ", lpwLinesTable[dwI]);
		if( blWriteString( hLog, pszText)== false)
			break;
	}
	wsprintf( pszText, "\r\n*** Voice lines coding table\r\n");
	blWriteString( hLog, pszText);
	for( dwI= 128; dwI< 256; dwI++)
	{	
		if((( dwI+1)%16)== 0) 
			wsprintf( pszText, "%04X\r\n", lpwLinesTable[dwI]);
		else
			wsprintf( pszText, "%04X ", lpwLinesTable[dwI]);
		if( blWriteString( hLog, pszText)== false)
			break;
	}
	CloseHandle( hLog);
	return true;
}


