#include <windows.h>
#include <stdio.h>
#include <string.h>
#include <conio.h>

#include "aldr9.h"
#include "main.h"

////////////////////////////////////////////////////////////////////

#include "OlhaFullFax.h"

#pragma comment( lib, "OlhaFullFax.lib" )

///////////////////////////////////////////////////////////////////////

void PrintUsage( char *ProgName)
{	
	char *Ptr;

	if(( Ptr= strrchr( ProgName, '\\'))== NULL)
		Ptr= ProgName;
	printf( "FAX sending simple test for Alder9 board (ISA&PCI) v1.0\n");
	printf( "Usage: %s [/B<BasePort>][/Q<IrqNum>][/L<Line>][/T][/I][/P][/T][/K<string>]<FileName.bmp>\n", Ptr);
	printf( "  /B<BasePort> Base port address in hex ( Find PCI board if not defined)\n");
	printf( "  /Q<IrqNum> IRQ number( 11 by default)\n");
	printf( "  /L<Line> Line number( 0 by default)\n");
	printf( "  /N<Digits> Dial string( 17 by default)\n");
	printf( "  /T Tone dial mode( by default)\n");
	printf( "  /P Pulse dial mode\n");
	printf( "  /I Internal extention line( by default)\n");
	printf( "  /E External town line\n");
	printf( "  /K<string> key for legal fax use\n");
	printf( "  <FileName.bmp> - BMP file for send to fax\n");
	printf( "\n\n");
}

DWORD ShowStat( AlderTransmitFaxData *afd)
{	
	DWORD NewStatFax;
	int param; 
	static int OldParam= 0;
	static DWORD OldStatFax= 0;
	char pszStatConnect[64]= "", pszRate[64]= "";

	NewStatFax= (DWORD)AlderGetTransmissionStatus( afd, &param);
	if(( OldParam== param) && ( OldStatFax== NewStatFax)) 
		return NewStatFax;
	OldStatFax= NewStatFax;
	OldParam= param;

	switch( NewStatFax)
	{	
		case FaxTransmitOff:
			strcpy( pszStatConnect,"end of transmision\n");
			lstrcpy( pszRate,"");
            break;
		case FaxTransmitConnecting:
			lstrcpy( pszStatConnect, "connecting\n");
			lstrcpy( pszRate,"");
			break;
		case FaxTransmitTraining:
			lstrcpy( pszStatConnect, "fax training\n");
			switch( param)
            {	case 0:
			        lstrcpy( pszRate,"2400");
					break;
				case 1:
					lstrcpy( pszRate,"4800");
					break;
				case 2:
					lstrcpy( pszRate,"7200");
					break;
				case 3:
					lstrcpy( pszRate,"9600");
					break;
            }
            break;
		case FaxTransmitSending:
			wsprintf( pszStatConnect, "sending page %d\n", param+1);
			break;
	}
	printf("  Status: %s  BaudRate: %s\n", pszStatConnect, pszRate);
	return NewStatFax;
}

void ShowFinalStatus( AlderTransmitFaxData *afd)
{
	switch( afd->dwFinalStatus)
	{	
		case FaxTransmitNoError:		
			printf("OK\n");
			break;
		case FaxTransmitSystemError:
			printf("There was a system error during transmit. GetLastError:%d\n",afd->dwSystemError);
			break;
		case FaxTransmitDriverError:			
			printf("The driver has returned a error number %04X\n",afd->dwDrvError);
			break;
		case FaxTransmitDeviceIoError:		
			printf("DeviceIOControl was finished unsuccessfully. GetLastError:%d\n",afd->dwSystemError);
			break;
		case FaxTransmitMemoryFault:			
			printf("Error at allocation of memory.\n");
			break;
		case FaxTransmitAlreadyStarted:		
			printf("The transmitter is already started.\n");
			break;
		case FaxTransmitFileReadError:
			printf("The input file is not found.\n");
			break;
		case FaxTransmitNoFaxPages:			
			printf("In an input file there are no facsimile sheets.\n");
			break;
		case FaxTransmitTooManyLines:
			printf("Too large number of a board / channel\n");
			break;
		case FaxTransmitDriverVersionFault:	
			printf("The wrong version of the driver.\n");
			break;
		case FaxTransmitFileCorrupt:
			printf("The input file is corrupted.\n");
			break;
		case FaxTransmitMustBeRun:
			printf("The transmitter should be already started.\n");
			break;
		case FaxTransmitUncorrectVersion:
			printf("Uncorrect DLL version.\n");
			break;
		default:
			printf("Unrecognized error.\n");
			break;
	}
}

void main( int argc, char **argv)
{	
	DWORD dwBasePort= 0, dwBoardHandle, dwIrqFreq= 9, dwIrq= 11;
	DWORD dwLine= 0;
	DWORD StartTicks, TotalTicks, dwI;
	DWORD dwBoardType, dwDllMajor, dwDllMinor;
	HANDLE hDrv;
	char *pszFileName, *pszKey;
	char pszDialString[32]= "14";
	bool blToneMode= true;
	bool blInternalLine= true;
	AlderTransmitFaxData afd;

	PrintUsage( argv[0]);

	if(( hDrv= hOpenDriver())== NULL)
	{	
		printf( " Unable open the driver. System error: %d\n\n", GetLastError());
		return;
	}
	blGetDriverVersion( hDrv);

	// Load command strings params
	if( argc> 1)
	{
		for( dwI= 1; dwI< (DWORD)argc; dwI++)
		{	
			// Get BasePort
			if( strncmp( argv[dwI], "/B", 2)== 0)
			{
				sscanf( argv[dwI]+ 2, "%X", &dwBasePort);
				if( blCheckBasePort( dwBasePort)== false)
				{
					printf( "Incorrect parameter \'Base Port: %04X\'\n", dwBasePort);
					return;
				}
				continue;
			}
			// Get IRQ number
			if( strncmp( argv[dwI], "/Q", 2)== 0)
			{
				dwIrq= atoi( argv[dwI]+2);
				if( blCheckLegalIrq( dwIrq)== false)
				{
					printf( "Incorrect parameter \'Irq: %d\'\n", dwIrq);
					return;
				}
				continue;
			}
			// Get Line number
			if( strncmp( argv[dwI], "/L", 2)== 0)
			{
				dwLine= atoi( argv[dwI]+ 2);
				if( dwLine> 15)
				{
					printf( "Incorrect parameter \'Line: %d\'\n", dwLine);
					return;
				}
				continue;
			}
			if( strncmp( argv[dwI], "/T", 2)== 0)
			{
				blToneMode= true;
				continue;
			}
			if( strncmp( argv[dwI], "/P", 2)== 0)
			{
				blToneMode= false;
				continue;
			}
			// Get Line Type - Town (external) or Extention (internal)
			if( strncmp( argv[dwI], "/I", 2)== 0)
			{
				blInternalLine= true;
				continue;
			}
			if( strncmp( argv[dwI], "/E", 2)== 0)
			{
				blInternalLine= false;
				continue;
			}
			if( strncmp( argv[dwI], "/N", 2)== 0)
			{
				lstrcpy( pszDialString, &argv[dwI][2]);
				continue;
			}
			pszFileName= argv[dwI];		
			if( strncmp( argv[dwI], "/K", 2)== 0)
			{
				pszKey= &argv[dwI][2];
				continue;
			}
		}
	}
	if( dwBasePort== 0)
	{	
		// Find PCI board
		if( blGetPciBoardInfo( hDrv, &dwBasePort, &dwBoardType)== false)
			goto EndProg;
		if( dwBasePort== 0)
		{	
			printf( "No Pci board found\n\n");
			goto EndProg;
		}
		printf( "PCI board found on %04X\n", dwBasePort);
	}
	else
	{	
		if( blGetBoardType( hDrv, dwBasePort, &dwBoardType)== false)
			goto EndProg;
		printf( "BoardType is: %04X\n", dwBoardType);
		if(( dwBoardType!= Alder9Board) && ( dwBoardType!= Alder10Board))
		{	
			printf( "Alder9 or Alder10 board not founded on port %04X\n", dwBasePort);
			goto EndProg;
		}
		switch( dwBoardType)
		{
			case Alder9Board:
				printf( "Alder9 ISA board found on %04X\n", dwBasePort);
				break;
			case Alder10Board:
				printf( "Alder10 ISA board found on %04X\n", dwBasePort);
				break;
			default:
				break;
		}
	}
	if( blOpenBoard( hDrv, dwBoardType, dwBasePort, &dwBoardHandle)== false)
		goto EndProg;
	if( blStartBoards( hDrv, dwIrq)== false)
		goto CloseBoard;
	StartTicks= GetTickCount();
	if( blInternalLine== false)
	{
		if( blSetPhoneParams( hDrv, dwBoardHandle, dwLine, 8, 1)== false)
			goto StopBoard;
	}
	else
	{
		if( blSetPhoneParams( hDrv, dwBoardHandle, dwLine, 4, 1)== false)
			goto StopBoard;
	}
	printf( "Selected line is %d\n", dwLine);
	AlderGetDllVersion( &dwDllMajor, &dwDllMinor);
	printf( " DLL version is %d.%d\n", dwDllMajor, dwDllMinor);
	printf( "Press ESC for exit...\n");

	///*******************************************************************
	//     AlderTransmitFaxData
	//  
	ZeroMemory( &afd, sizeof( AlderTransmitFaxData));
	afd.cbSize= sizeof( AlderTransmitFaxData);
	//  
	lstrcpy( afd.pszKey, pszKey);
	//  ,     
	afd.hcDrv= hDrv;
	//  
    afd.dwBoardHandle= dwBoardHandle;
	//      
	afd.dwLine= dwLine;
	//    -
	afd.EndFaxTransmitFunc= NULL;
	//       
	afd.dwUserMessNumberStop= 0;
///*******************************************************************

	//      
	// BMP     
	GetCurrentDirectory( sizeof( afd.pszFaxFileName), afd.pszFaxFileName);
	GetTempFileName( afd.pszFaxFileName, "fax", 0, afd.pszFaxFileName);

	//  BMP 
	if( bmp2fax( afd.pszFaxFileName, pszFileName))
	{	
		printf("Wrong a BMP  file.\n");
		goto StopBoard;
	}
	if( blSetHook( hDrv, dwBoardHandle, dwLine, true)== false)
		goto StopBoard;
	if( blWaitSignal( hDrv, dwBoardHandle, dwLine, drvAlderLineDialTone, 10)== false)
		goto HookOff;
	if( blDial( hDrv, dwBoardHandle, dwLine, pszDialString, blToneMode)== false)
		goto HookOff;
	if( blWaitSignal( hDrv, dwBoardHandle, dwLine, drvAlderLineFAX, 30)== false)
		goto HookOff;
	AlderOpenFaxTransmission( &afd);
	do
	{	
		//    (BUSY)
		if( blCheckSignal( hDrv, dwBoardHandle, dwLine, drvAlderLineBusy)== true)
		{
			printf( " BUSY found!\n");
			break;
		}
		printf("\r");
		if( kbhit()!=0) 
		{	
			if( getch()== 0x1B)
				break;
		}
		Sleep(50);
	} while( ShowStat( &afd)!= FaxTransmitOff);
	ShowFinalStatus( &afd);
	AlderCloseFaxTransmission( &afd);
	printf( "EndOfTransmission\n\n");
	blWaitSignal( hDrv, dwBoardHandle, dwLine, drvAlderLineBusy, 20);
HookOff:
	blSetHook( hDrv, dwBoardHandle, dwLine, false);
	Sleep(100);
StopBoard:
	DeleteFile( afd.pszFaxFileName);
	blStopBoards( hDrv);
CloseBoard:
	blCloseBoard( hDrv, dwBoardHandle);
	TotalTicks= GetTickCount()- StartTicks;
	printf( "Total processing time: %d.%d\n", TotalTicks/1000, TotalTicks%1000);
EndProg:
	CloseDriver( hDrv);
}
